/*
 * RC Servo driver for VoCore
 *
 * Copyright (C) 2015 Ton Augustin <ton@augustin.com>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This is the servos.h for /drivers/servos
 */
#ifndef __SERVOS_H_INCLUDED
#define __SERVOS_H_INCLUDED

#include <linux/device.h>
#include <linux/rwsem.h>
#include <linux/leds.h>

static inline void __servo_set_min_pulsewidth(struct servo_classdev *servo_cdev, int value)
{
	if (value < SERVO_MIN)
		value = SERVO_MIN;
	servo_cdev->min_pulsewidth = value;
}

static inline int servo_get_min_pulsewidth(struct servo_classdev *servo_cdev)
{
	return servo_cdev->min_pulsewidth;
}

static inline void __servo_set_pulsewidth(struct servo_classdev *servo_cdev, int value)
{
	if( value != 0) {
		if (value > servo_cdev->max_pulsewidth)
			value = servo_cdev->max_pulsewidth;
		if (value < servo_cdev->min_pulsewidth)
			value = servo_cdev->min_pulsewidth;
	}		
	servo_cdev->pulsewidth = value;
	// set timeout if applicable
	if(servo_cdev->timeoutactive) servo_cdev->timeoutcnt = servo_cdev->timeout;
}

static inline int servo_get_pulsewidth(struct servo_classdev *servo_cdev)
{
	return servo_cdev->pulsewidth;
}


static inline void __servo_set_max_pulsewidth(struct servo_classdev *servo_cdev, int value)
{
	if (value > SERVO_MAX)
		value = SERVO_MAX;
	servo_cdev->max_pulsewidth = value;
}

static inline int servo_get_max_pulsewidth(struct servo_classdev *servo_cdev)
{
	return servo_cdev->max_pulsewidth;
}


static inline void __servo_set_timeout(struct servo_classdev *servo_cdev, int value)
{
	if (value > MAX_TIMEOUT)
		value = MAX_TIMEOUT;
	servo_cdev->timeout = value / MS_TIME_TO_FRAMES;
	servo_cdev->timeoutactive 	= (servo_cdev->timeout ? 1 : 0 );	
}

static inline int servo_get_timeout(struct servo_classdev *servo_cdev)
{
	return servo_cdev->timeout;
}




#endif	/* __SERVOS_H_INCLUDED */
