/*
 * RC Servo driver for VoCore
 *
 * Copyright (C) 2015 Ton Augustin <ton@augustin.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This is the servo.h for /include/linux
 */
#ifndef __LINUX_SERVOS_H_INCLUDED
#define __LINUX_SERVOS_H_INCLUDED

#include <linux/list.h>
#include <linux/spinlock.h>
#include <linux/rwsem.h>
#include <linux/timer.h>

struct device;
/// servo times in 10us, gives 50 to 250 range, 8 bits
#define	SERVO_OFF		0
#define	SERVO_MIN		50
#define	SERVO_MID		150
#define	SERVO_MAX		250
// Frametime is 25ms, also in 10ns
#define 	FRAMETIME		2500
// sync time in ms. 
#define	MAX_SYNC_TIME		10000
#define	MIN_SYNC_TIME		500
#define	MS_TIME_TO_FRAMES	25
#define 	MAX_TIMEOUT		60000

struct servo_classdev {
	const char	*name;
	
	int	pulsewidth;			// pulsewidth when active and not timed out
	int	max_pulsewidth;		// Minimum value (can never be less than SERVO_MIN)
	int	min_pulsewidth;		// Minimum value (can never be less than SERVO_MIN)

	/* timeout, number of msec the pulse will live after setting */
	int	timeout;
	int 	timeoutcnt;			// recharged to timeout every write, decreased every frame
	int 	timeoutactive;		// 1 = yes, 0 is no timeout

	/* Sync vars, synchronised motion of multiple servos */
	int	syncactive;			// operating within this sync?
	int	synctime;			// total number of frames while syncing
	int 	syncstartPW;			// Start value Pulse Width
	int 	syncendPW;			// End value Pulse Width
	int 	syncdeltaPW;			// Difference to span of Pulse Width
	int 	syncdirectionisup;		// Going up 1, or down 0
	int 	synccurrenttime;

	int		 flags;
#define SERVO_SUSPENDED		(1 << 0)	// Lower 16 bits reflect status 
#define SERVO_CORE_SUSPENDRESUME	(1 << 16)	// Upper 16 bits reflect control information 

	struct device			*dev;
	
};

extern int  servo_classdev_register(struct device *parent, struct servo_classdev *servo_cdev);
extern void servo_classdev_unregister(struct servo_classdev *servo_cdev);
extern void servo_classdev_suspend(struct servo_classdev *servo_cdev);
extern void servo_classdev_resume(struct servo_classdev *servo_cdev);

struct gpio_servo_data {
	struct servo_classdev cdev;
	unsigned gpio;
	u8 active_low;
};

struct gpio_servos_priv {
	struct rt_timer *rtimer;
	int num_servos;
	struct gpio_servo_data servo[];
};


/*
 * Generic SERVO platform data
 */
struct servo_info {
	const char	*name;
	int		flags;
};

struct servo_platform_data {
	int		num_servos;
	struct servo_info	*servos;
};



struct gpio_servo {
	const char 			*name;
	unsigned 			gpio;
	int				timeout;
	int				min_pulsewidth;
	int 				max_pulsewidth;
	unsigned			active_low : 1;
};


struct gpio_servo_platform_data {
	int 		num_servos;
	const struct gpio_servo *servo;

};

struct platform_device *gpio_servo_register_device( int id, const struct gpio_servo_platform_data *pdata);


#endif		/* __LINUX_SERVOS_H_INCLUDED */
